package reittiLoki;

import java.io.PrintStream;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author OMISTAJA
 * @version 1.4.2025
 *
 */
public class Halli {
    private int id = 0;
    String nimi = "";
    String osoite = "";
    int suoritusNro;
    
    private static int seuraavaNro = 1;
    
    
    /**
     * Alustaa Hallin tiedot.
     */
    public Halli() {
        
    }
      
    
   /**
    * Tulostetaan suorituksen tiedot
    * @param out Minne tulostetaan.
    */
    public void tulosta(PrintStream out) {
       out.println("Id:" + id + " \n" + "Halli: " + nimi + " \n" + osoite + " \n" + "linkitetty suoritus: " + suoritusNro);
       out.println();
   }
    
    /**
     * Palauttaa suorituksen Id:n.
     * @return Suorituksen Id.
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.taytaHalli(1);
     * eka.getId() === 0;
     * </pre>
     */
    public int getId() {
        return id;
    }
    
    /**
     * Palauttaa suorituksen Id:n.
     * @return Suorituksen Id.
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.taytaHalli(1);
     * eka.getSuoritusNro() === 1;
     * </pre>
     */
    public int getSuoritusNro() {
        return suoritusNro;
    }
    
    /**
     * Aliohjelma antaa suoritukselle seuraavan id:n.
     * @example
     * <pre name="test">
     * Suoritus testi1 = new Suoritus();
     * testi1.getId() === 0;
     * testi1.rekisteroi();
     * Suoritus testi2 = new Suoritus();
     * testi2.rekisteroi();
     * int n1 = testi1.getId();
     * int n2 = testi2.getId();
     * n1 === n2-1;
     * </pre>
     */
    public void rekisteroi() {
        this.id = seuraavaNro;
        seuraavaNro++;
    }
    
    /**
     * Apumetodi, jolla täytetään tiedot hallille. Käytetään kehitysvaiheessa ja testeissä.
     * Poistetaan myöhemmin.
     * @param suoritus Sen suorituksen ID-numero johon halli liitetään.
     */
    public void taytaHalli(int suoritus) {
        
        nimi = "Ristikko"; // + tarkistusNro(); Käytetään jos tarvii arpoa tarkistusnro nimen perään.
        osoite = "Testikatu 8";
        this.suoritusNro = suoritus;
    }
    
    /**
     * Tarkistusnumeroa käytetään vain varmistamaan että suoritukset eroavat toisistaan. Poistetaan kehityksen jälkeen tarvittaessa.
     * @return Tarkistusnumero
     */
    public int tarkistusNro() {
        double nro = Math.random() * 100;
        int tulos = (int) nro;
        return tulos;
    }
    
    /**
     * @return palauttaa suorituksen nimen.
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.taytaHalli(1);
     * eka.getNimi() === "Ristikko";
     * </pre>
     */
    public String getNimi() {
        return this.nimi;
    }
    
    /**
     * Muuntaa Suorituksen tiedot merkkijonoksi.
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.taytaHalli(2);
     * eka.toString() === "0|Ristikko|Testikatu 8|2";
     * </pre>
     * </pre>
     */
    @Override
    public String toString() {
        return "" +
                getId() + "|" +
                nimi + "|" +
                osoite + "|" +
                suoritusNro;          
    }
    
    /**
     * @param r Tutkittava rivi
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * String s = "23|Ristikko|Konalantie 12|3";
     * eka.parse(s);
     * eka.getId() === 23;
     * eka.getNimi() === "Ristikko";
     * eka.getOsoite() === "Konalantie 12";
     * eka.getSuoritusNro() === 3;
     * </pre>
     */
    public void parse(String r) {
           StringBuilder sb = new StringBuilder(r);
           setId(Mjonot.erota(sb, '|', getId()));
           nimi = Mjonot.erota(sb, '|', nimi);
           osoite = Mjonot.erota(sb, '|', osoite);
           suoritusNro = Mjonot.erota(sb, '|', suoritusNro);

       }

    /**
     * Asettaa halutun id:n hallille.
     * @param nro asetettava id
     */
    private void setId(int nro) {
        id = nro;
        if(id >= seuraavaNro) seuraavaNro = id +1;
        
    }
    
    /**
     * Lisätään Halliin suorituksen numero.
     * @param nro Linkitetyn suorituksen ID-numero
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.setSuoritusNro(1);
     * eka.getSuoritusNro() === 1;
     * </pre>
     */
    public void setSuoritusNro(int nro) {
        suoritusNro = nro;      
    }


   /**
    * Main ohjelmaa käytetään vain Halli tiedostoa testatessa. Sen takia kommenteissa.
     * @param args ei käytössä.
     */
    /*   public static void main(String[] args) {
        Halli kala = new Halli();
        kala.taytaHalli(1);
        kala.tulosta(System.out);
    }
    */


    /**
     * Asettaa hallille nimen.
     * @param s Haluttu nimi
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.setNimi("Toimii");
     * eka.getNimi() === "Toimii";
     * </pre>
     */
    public void setNimi(String s) {
        this.nimi = s;
        return;
        
    }


    /**
     * Asettaa hallille osoitteen.
     * @param s Haluttu osoite
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.setOsoite("Toimii");
     * eka.getOsoite() === "Toimii";
     * </pre>
     */
    public void setOsoite(String s) {
        this.osoite = s;
        return;
        
    }


    /**
     * Hakee hallin osoitteen.
     * @return Hallin osoite
     * @example
     * <pre name="test">
     * Halli eka = new Halli();
     * eka.taytaHalli(1);
     * eka.getOsoite() === "Testikatu 8";
     * </pre>
     */
    public String getOsoite() {
        return this.osoite;
        
    }
}
