package reittiLoki;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Scanner;

/**
 * @author OMISTAJA
 * @version 1.4.2025
 *
 */
public class Hallit {
    
   private Collection<Halli> alkiot = new ArrayList<Halli>();
   private String tiedostoNimi = "";
    
 /**
 * Hallit alustetaan Hallit luokka.
 */
public Hallit() {
       
   }
    /**
     * @param ha lisättävä halli.
     * @example
     * <pre name="test">
     * Hallit hallit = new Hallit();
     * Halli eka = new Halli();
     * eka.taytaHalli(2);
     * hallit.lisaa(eka);
     * hallit.annaHallit(2).equals(hallit);
     * </pre>
     */
    public void lisaa(Halli ha) {
        alkiot.add(ha);
    }
    
    /**
     * @param tunnusNro Hallin tunnusnumero jolla suorituksia haetaan.
     * @return palauttaa suoritukset tunnusnumeron perusteella.
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     * #import java.util.Collection;
     * Hallit hallit = new Hallit();
     * Hallit hallit2 = new Hallit();
     * Halli eka = new Halli();
     * eka.taytaHalli(1);
     * hallit.lisaa(eka);
     * hallit2.lisaa(eka);
     * Halli toka = new Halli();
     * toka.taytaHalli(2);
     * hallit.lisaa(toka);
     * Halli kolmas = new Halli();
     * kolmas.taytaHalli(1);
     * hallit.lisaa(kolmas);
     * hallit2.lisaa(kolmas);
     * hallit2.equals(hallit.annaHallit(1));
     * </pre>
     */
    public List<Halli> annaHallit(int tunnusNro){
        List<Halli> palautetut = new ArrayList<Halli>();
        
        for(Halli ha: alkiot)
            if (ha.getSuoritusNro() == tunnusNro) palautetut.add(ha);
        return palautetut;
    }
    

    
    /**
     * @param hakem Hakemisto, josta käytettäävää tiedostoa yritetään hakea.
     * @throws SailoException Virhetilanne mikäli tiedostoa ei löydy.
     */
    public void lueTiedostosta(String hakem) throws SailoException{
        tiedostoNimi = hakem + "/hallit.dat";
        File tied = new File(tiedostoNimi);
        
        try (Scanner fi = new Scanner (new FileInputStream(tied))){
            while (fi.hasNext()) {
                String seuraava = fi.nextLine();
                if (seuraava == null || "".equals(seuraava) || seuraava.charAt(0) == ';') continue;
                Halli halli = new Halli();
                halli.parse(seuraava);
                lisaa(halli);
            }
        }   catch (FileNotFoundException e) {
                throw new SailoException("Ei saada luettua tiedostoa");
        }
        }
    /**
     * @param hakem Hakemiston nimi
     * @throws SailoException Kertoo mikäli törmätään virhetilaan tiedoston tallantemisessa.
     */
    public void tallenna(String hakem) throws SailoException {
        File tied = new File(hakem + "/hallit.dat");
        try (PrintStream fo = new PrintStream(new FileOutputStream(tied))){
            for (Halli halli: alkiot) {               
                fo.println(halli.toString());
            }
        } catch (FileNotFoundException e) {
        throw new SailoException ("Tiedoston avaaminen ei vielä toimi");
        }
        }
    
    /**
     * Poistaa suoritukseen liitetyt hallit.
     * @param s Suoritus johon liitetyt hallit poistetaan.
     * @example
     * <pre name="test">
     * #import java.util.ArrayList;
     * #import java.util.Collection;
     * Suoritus eka = new Suoritus();
     * eka.taytaSuoritus();
     * Hallit hallit2 = new Hallit();
     * Hallit hallit = new Hallit();
     * Halli testi = new Halli();
     * testi.taytaHalli(0);
     * hallit.poistaHallit(eka);
     * hallit.lisaa(testi);
     * hallit.equals(hallit2);
     * </pre>
     */
    public void poistaHallit(Suoritus s) {
        Collection<Halli> alkiot2 = new ArrayList<Halli>();
        int id = s.getId();
        
        for(Halli a : alkiot) {
           if(a.getSuoritusNro() != id) alkiot2.add(a);
        }
        alkiot = alkiot2;
    }
    
    
    /**
     * Main ohjelmaa käytetään vain Hallit tiedostoa testatessa. Kommentoitu piiloon.
     * @param args ei käytössä.
     * @throws SailoException Kertoo mikäli Suorituksia ei enää mahdu taulukkoon.
     */
 /*   public static void main(String[] args) throws SailoException {
        
     Hallit hallit = new Hallit();
     
     try {
         hallit.lueTiedostosta("hallit");
     } catch (SailoException ex) {
         System.err.println(ex.getMessage());
     }
     
     Halli kala = new Halli();
     kala.rekisteroi();
     kala.taytaHalli(1);
     Halli salmis = new Halli();
     salmis.rekisteroi();
     salmis.taytaHalli(2);
     
     hallit.lisaa(kala);
     hallit.lisaa(salmis);
     
       System.out.println("================ Hallit testi ================");
     
     List<Halli> hallit2 = hallit.annaHallit(1);
     
     for (Halli ha : hallit2) {
        System.out.print(ha.getSuoritusNro() + " ");
         ha.tulosta(System.out);     
     }
     try {
         hallit.tallenna("hallit");
     } catch (SailoException e) {
         System.err.println(e.getMessage());
     }
    }
    */
}


