package reittiLoki;

import java.io.File;
import java.util.List;

/**
 * @author OMISTAJA
 * @version 31.3.2025
 *
 */
public class Reittiloki {
    
    private Suoritukset suoritukset = new Suoritukset();
    private Hallit hallit = new Hallit();

    /**
     * Aliohjelmalla lisätään suoritus Lokiin.
     * @param suoritus lisättävä suoritus
     * @throws SailoException Virhetila jos lisääminen epäonnistuu.
     */
    public void lisaa(Suoritus suoritus) throws SailoException {
        suoritukset.lisaa(suoritus);
    }
    
    /**
     * Lisätään uusi halli valittavaksi.
     * @param ha halli
     */
    public void lisaa(Halli ha) {
        hallit.lisaa(ha);
    }
    
    /**
     * @return palauttaa suoritusten lukumäärän.
     */
    public int getSuorituksia() {
        return suoritukset.getLkm();
    }
    
    /**
     * Antaa Lokin suorituksen annetun indeksin perusteella.
     * @param i Monesko suoritus halutaan.
     * @return Suoritus paikasta i
     */
    public Suoritus annaSuoritus(int i) {
        return suoritukset.anna(i);
    }
    
    /**
     * @param s suoritus
     * @return listaus halleista
     */
    public List<Halli> annaHallit(Suoritus s) {
        return hallit.annaHallit(s.getId());
    }
    
    /**
     * Tallentaa suorituksen tiedot tiedostoon.
     * @param hakem Reittiloki hakemiston nimi
     * @throws SailoException Näytetää mikäli ongelmia tallentamisessa.
     */
    public void tallenna(String hakem) throws SailoException {
        String vi = "";
        try {
        suoritukset.tallenna(hakem);
        } catch (SailoException e) {
            vi= e.getMessage();
        }        
        try {
        hallit.tallenna(hakem);
    } catch (SailoException e) {
        vi += e.getMessage();
    }
        if (!"".equals(vi)) throw new SailoException(vi);
    }
    
    /**
     * Lukee lokin tiedot tiedostoista.
     * @param nimi nimi josta tiedostosta luetaan.
     * @throws SailoException Virhetilanne mikäli tiedostoja ei onnistuta lukemaan.
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        File tied = new File(nimi);
        tied.mkdir();
        suoritukset = new Suoritukset();
        hallit = new Hallit();
        
        suoritukset.lueTiedostosta(nimi);
        hallit.lueTiedostosta(nimi);
    }
    
    /**
     * Korvaa samalla id:llä olevan suorituksen.
     * @param suoritus Suoritus joka tulee korvaamaan olemassa olevan Suorituksen.
     */
    public void korvaaLisaa(Suoritus suoritus){
        suoritukset.korvaaLisaa(suoritus);
        
    }
    
    /**
     * Ohjaa ohjelman poistamaan sekä suorituksen, että siihen liitetyt hallit.
     * @param s Ohjelmasta poistettava suoritus.
     */
    public void poista(Suoritus s) {
        hallit.poistaHallit(s);
        suoritukset.poistetaan(s);
    }
    

    /**
     * @param args ei käytössä.
     * @throws SailoException Lisäyksen virhetila, jos lisääminen ei onnistu.
     */
    public static void main(String[] args) throws SailoException {
        Reittiloki loki = new Reittiloki();
        
        Suoritus eka = new Suoritus();
        Suoritus toka = new Suoritus();
        eka.rekisteroi();
        eka.taytaSuoritus();
        toka.rekisteroi();
        toka.taytaSuoritus();
        
        loki.lisaa(eka);
        loki.lisaa(toka);
        
        Hallit hallit = new Hallit();
        Halli kala = new Halli();
        kala.taytaHalli(1);
        Halli salmis = new Halli();
        salmis.taytaHalli(toka.getId());
        
        hallit.lisaa(kala);
        hallit.lisaa(salmis);
        
        for(int i = 0; i<loki.getSuorituksia(); i++) {
            Suoritus suoritus = loki.annaSuoritus(i);
            suoritus.tulosta(System.out);
            List<Halli> hallit2 = hallit.annaHallit(suoritus.getId());       
            for (Halli ha : hallit2) {
            System.out.print(ha.getSuoritusNro() + " ");
            ha.tulosta(System.out);
            
        }

    }
    }

}


