package reittiLoki;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.Scanner;
import java.io.FileInputStream;

/**
 * @author OMISTAJA
 * @version 31.3.2025
 *
 */
public class Suoritukset {

    private static final int Maksimi = 5;
    private boolean muutettu = false;
    private int lkm;
    private Suoritus[] alkiot;
    private String tiedostoNimi = "";
    
  /**
 * Alustetaan Suoritukset taulukko.
 */
public Suoritukset() {
      this.alkiot = new Suoritus[Maksimi];
  }
  
  /**
 * @param suoritus lisättävän suorituksen tieto.
 * @throws SailoException Kertoo mikäli suorituksia on liikaa.
 * @example
 * <pre name="test">
 * Suoritukset suoritukset = new Suoritukset();
 * Suoritus eka = new Suoritus();
 * eka.taytaSuoritus();
 *  try {
 *  suoritukset.lisaa(eka);
 *  }catch (SailoException e){
 *  //
 *  }
 * suoritukset.anna(0).getNimi() === "Kattoralli";
 * </pre>
 */
public void lisaa(Suoritus suoritus) throws SailoException{
     if(lkm >= alkiot.length) {
         Suoritus[] alkiot2 = Arrays.copyOf(alkiot, alkiot.length*2);
         alkiot = alkiot2;
         muutettu = true;
     }
     this.alkiot[this.lkm] = suoritus;
     this.lkm++;
     muutettu = true;
  }



/**
 * @param i Monesko Suoritus halutaan tarkistaa.
 * @return Valitun Suorituksen.
 * @throws IndexOutOfBoundsException Antaa virheen mikäli kysyttyä alkiota ei ole taulukossa.
 * @example
 * <pre name="test">
 * Suoritukset suoritukset = new Suoritukset();
 * Suoritus eka = new Suoritus();
 * eka.taytaSuoritus();
 *  try {
 *  suoritukset.lisaa(eka);
 *  }catch (SailoException e){
 *  //
 *  }
 *  suoritukset.anna(0).equals(eka);
 * </pre>
 */
public Suoritus anna(int i)throws IndexOutOfBoundsException {
    if (i < 0 || this.lkm <= i)
        throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
    return alkiot[i];
}
  
  /**
 * @return palauttaa Suoritusten lukumäärän.
 * @example
 * <pre name="test">
 * Suoritukset suoritukset = new Suoritukset();
 * Suoritus eka = new Suoritus();
 * eka.taytaSuoritus();
 * suoritukset.getLkm() === 0;
 * try {
 *  suoritukset.lisaa(eka);
 *  }catch (SailoException e){
 *  //
 *  }
 * suoritukset.getLkm() === 1;
 * </pre>
 */
public int getLkm() {
      return this.lkm;
  }

/**
 * @param hakem Hakemisto, josta käytettäävää tiedostoa yritetään hakea.
 * @throws SailoException Virhetilanne mikäli tiedostoa ei löydy.
 */
public void lueTiedostosta(String hakem) throws SailoException{
    tiedostoNimi = hakem + "/suoritukset.dat";
    File tied = new File(tiedostoNimi);
    
    try (Scanner fi = new Scanner (new FileInputStream(tied))){
        while (fi.hasNext()) {
            String seuraava = fi.nextLine();
            if (seuraava == null || "".equals(seuraava) || seuraava.charAt(0) == ';') continue;
            Suoritus suoritus = new Suoritus();
            suoritus.parse(seuraava);
            lisaa(suoritus);
        }
    }   catch (FileNotFoundException e) {
            throw new SailoException("Ei saada luettua tiedostoa");
    }
}

/**
 * @param hakem Hakemiston nimi
 * @throws SailoException Kertoo mikäli törmätään virhetilaan tiedoston tallantemisessa.
 */
public void tallenna(String hakem) throws SailoException {
    if (!muutettu) return;
    File tied = new File(hakem + "/suoritukset.dat");
    try (PrintStream fo = new PrintStream(new FileOutputStream(tied))){
        for (int i = 0; i < this.getLkm(); i++) {
            Suoritus suoritus = this.anna(i);
            if(suoritus == null) continue;
            fo.println(suoritus.toString());
        }
    } catch (FileNotFoundException e) {
    throw new SailoException ("Tiedoston avaamista ei ole vielä tehty");
    }
    muutettu = false;
}

/**
 * @param suoritus Korvaava suoritus
 */
public void korvaaLisaa(Suoritus suoritus){
    int id = suoritus.getId();
    
    for(int i = 0; i < lkm; i++) {
        if (alkiot[i].getId() == id) {
            alkiot[i] = suoritus;
            muutettu = true;
            return;
        }
    }
    try {
        lisaa(suoritus);
    } catch (SailoException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
    }
}

/**
 * Aliohjelmaa poistaa suorituksent taulukosta.
 * @param s Poistettava suoritus
 * @example
 * <pre name="test">
 * Suoritukset suoritukset = new Suoritukset();
 * Suoritus eka = new Suoritus();
 * eka.taytaSuoritus();
 * try {
 *  suoritukset.lisaa(eka);
 *  }catch (SailoException e){
 *  //
 *  }
 *  int luku = suoritukset.getLkm();
 *  suoritukset.poistetaan(eka);
 *  suoritukset.getLkm() === luku - 1;
 * </pre>
 */
public void poistetaan(Suoritus s){
    
    Suoritus[] alkiot2 = new Suoritus[alkiot.length];
    int id = s.getId()-1;
    
    for (int i = 0, k = 0; i < alkiot.length; i++) {
        if(i == id) continue;
        alkiot2[k] = alkiot[i];
        k++;
    }
    alkiot = alkiot2;
    this.lkm--;   
}




/**
 * @param args ei käytössä.
 * @throws SailoException Kertoo mikäli Suorituksia ei enää mahdu taulukkoon.
 */
/*public static void main(String[] args) throws SailoException {
    
 Suoritukset suoritukset = new Suoritukset();
 
 try {
     suoritukset.lueTiedostosta("suoritukset");
 } catch (SailoException ex) {
     System.err.println(ex.getMessage());
 }
 
 Suoritus eka = new Suoritus();
 Suoritus toka = new Suoritus();
 eka.taytaSuoritus();
 toka.taytaSuoritus();
 eka.rekisteroi();
 toka.rekisteroi();
 
 suoritukset.lisaa(eka);
 suoritukset.lisaa(toka);
 
 System.out.println("================ Suoritukset testi ================");
 
 for(int i = 0; i < suoritukset.getLkm(); i++ ) {
     Suoritus suoritus = suoritukset.anna(i);
     System.out.println("Suorituksen indeksiluku on " + i);
     suoritus.tulosta(System.out);
     
 }
 
 try {
     suoritukset.tallenna("suoritukset");
 } catch (SailoException e) {
     System.err.println(e.getMessage());
 }
 
 suoritukset.getLkm();
    

}
*/
}


